import { define } from './define';
describe('Whole functionality test', function () {
    it('Should create stylesheet with only dynamic styles creator', function () {
        var styleSheet = define(function () { return ({
            something: {
                backgroundColor: 'green'
            }
        }); });
        expect(typeof styleSheet.dynamic).toBe('function');
        expect(styleSheet.staticStyles).toBeFalsy();
    });
    it('Should create stylesheet with only static styles', function () {
        var styleSheet = define({
            something: {
                backgroundColor: 'green'
            }
        });
        expect(styleSheet.staticStyles).toEqual({
            something: {
                backgroundColor: 'green'
            }
        });
        expect(styleSheet.dynamic).toBeFalsy();
    });
    it('Should create stylesheet with both dynamic and static styles', function () {
        var styleSheet = define(function () { return ({
            someField: {
                color: 'red'
            }
        }); }, {
            something: {
                backgroundColor: 'green'
            }
        });
        expect(styleSheet.staticStyles).toEqual({
            something: {
                backgroundColor: 'green'
            }
        });
        expect(typeof styleSheet.dynamic).toBe('function');
        expect(styleSheet.dynamic({}, {}));
    });
    it('Dynamic() should generate styles depending on passed theme and prop', function () {
        var styleSheet = define(function (_a, _b) {
            var primary = _a.primary;
            var backgroundColor = _b.backgroundColor;
            return ({
                root: {
                    color: primary,
                    backgroundColor: backgroundColor
                }
            });
        });
        expect(styleSheet.dynamic({ primary: 'green' }, { backgroundColor: 'red' })).toEqual({
            root: {
                color: 'green',
                backgroundColor: 'red'
            }
        });
    });
});
//# sourceMappingURL=define.spec.js.map